# Adding a Dynamic Domain Name System (DDNS) provider
Thanks to rust's type guarantees, it is easy to add a provider without messing things up, cause if you do, the compiler will tell you!

For this example, I will go through the process of adding [Dynv6](https://dynv6.com) as a provider, but the general process will be the same for most providers.

## Creating the provider type
To start off, create a file under `src/providers/<provider>.rs` and enter the following content:
```rust
#[derive(serde::Deserialize)]
pub(crate) struct ProviderCamelCase {} // Replace with name of provider

```

Fill the struct with the content that you will need from the config file, for example:

```rust
#[derive(serde::Deserialize)]
pub(crate) struct Dynv6 {
    domains: Vec<String>,
    token: String,
    ip_type: Option<IpType>,
}
```

If you have rust analyzer installed, it should be suggesting to add the file to the module tree. Go ahead and execute the code action now. If you do not have rust analyzer yet, install it, as it will make your life much easier.

Next, implement the `Provider` trait for your provider:

```rust
use super::Provider;

// -- snip --

impl Provider for Dynv6 {}
```

Execute the code action to implement missing members to get the required functions and their signatures.

## Implementing the functions
### check_config()
First, we will implement the `check_config()` function. The purpose of this function is to check if the values in the config file are witihin the allowed range. You do not have to check if they adhere to the types set in the struct earlier, as that is checked by rust beforehand.

If you are adding checks, make sure to add the type of error to the `Error` enum in `src/error.rs`, rust analyzer will tell you that you will have to add that varient to the `match` statement below the enum.

In our case, no checks specific to Dynv6 are needed, so we'll continue.

### init_defaults()
The `init_defaults()` function allows for you to set values from the `defaults` table to the config for your provider if the user has not specified a value for your provider. Possible values you can use can be found in the `Defaults` struct in `src/config.rs`

For our config, the field `ip_type` is shared with the default config, so we will get that value if it is not found:

```rust
fn init_defaults(&mut self, defaults: &crate::config::Defaults) {
    if self.ip_type.is_none() {
        self.ip_type = Some(defaults.ip_type)
    };
}
```

### update()
The last (but most important) function we are going to implement is the `update()` function. This is what actually updates the dns records. It executes requests to update records and returns the results, along with the (sub)domain which was being updated and the ip type(s).

```rust
fn update(
    &self,
    v4: Option<Ipv4Addr>,
    v6: Option<Ipv6Addr>,
) -> Vec<(
    Result<reqwest::blocking::Response, reqwest::Error>,
    String,
    IpType,
)> {
    // Here is a vec of the requests to return
    let mut requests = vec![];

    // Here we are getting the ips to set depending on what ips types are set to be updated as according to our config file. This is a vector of tuples that will be passed into our request as query parameters.
    let ips = match self
        .ip_type
        .expect("Should be overwritten by init_defaults")
    {
        IpType::Both => (
            Some(("ipv6", v6.map(|x| x.to_string().clone()))),
            Some(("ipv4", v4.map(|x| x.to_string().clone()))),
        ),
        IpType::V6 => (Some(("ipv6", v6.map(|x| x.to_string().clone()))), None),
        IpType::V4 => (Some(("ipv4", v4.map(|x| x.to_string().clone()))), None),
    };

    for domain in &self.domains {
        let client = reqwest::blocking::Client::new();
        requests.push((
            client.execute(
                client
                    .get("https://dynv6.com/api/update")
                    // Here we are setting query values from our config
                    .query(&[
                        Some(("zone", Some(domain.clone()))),
                        Some(("token", Some(self.token.clone()))),

                        ips.0.clone(),
                        ips.1.clone(),
                    ])
                    .build()
                    .expect("Reqwest Documentation does not state when this could fail"),
            ),
            // Here we are adding the domain which we are updating so that it can be displayed in standard output
            domain.clone(),
            // Here we are adding the ip type so it can be displayed in standard output
            self.ip_type.expect("Must be Some() due to init_defaults()"),
        ));
    }

    // Lastly, we return our vec of requests
    requests
}
```
