use crate::providers::Provider;
mod config;
mod error;
mod ips;
mod providers;
use clap::{arg, command, value_parser, ArgAction, Command};
use core::panic;
use ips::{get_ip, IpType};
use std::{
    fs,
    net::{Ipv4Addr, Ipv6Addr},
    path::PathBuf,
    process, thread,
    time::Duration,
};

fn main() {
    let command = command!()
        .arg(
            arg!(-c --config <FILE> "Sets a custom config file")
            .value_parser(value_parser!(PathBuf)).default_value("/etc/ddnrs.toml").global(true),
        )
        .subcommand(Command::new("run").about("Starts updating dns records")
            .arg(arg!(-o --once ... "Updates dns records once, then quits").action(ArgAction::SetTrue))
            .arg(arg!(
                -s --scan ... "Scans config file for updates before each update, rather than only scanning once when the program starts"
            ).action(ArgAction::SetTrue))
        )
        .subcommand(Command::new("check").about("Checks the sanity of the config file"))
        .subcommand_required(true);
    let matches = command.get_matches();

    match matches.subcommand_name() {
        Some("check") => {
            get_checked_config(matches.get_one::<PathBuf>("config"));
            println!("Everything's in order")
        }
        Some("run") => {
            let sub_matches = matches.subcommand_matches("run").expect("Static argument");
            let once = sub_matches.get_flag("once");
            let scan = sub_matches.get_flag("scan");

            let mut config = get_checked_config(matches.get_one::<PathBuf>("config"));

            let mut ips = get_ip();

            print_addresses(ips.0, ips.1);

            if ips != (None, None) {
                let result = update_records(&config, ips.0, ips.1);
                if let Err(e) = result {
                    eprintln!("Error while updating domain : {e}")
                }
            }

            if !once {
                loop {
                    thread::sleep(Duration::from_secs(config.interval));

                    let tmp = get_ip();

                    print_addresses(tmp.0, tmp.1);
                    if ((ips.0 != tmp.0 && tmp.0.is_some()) || (ips.1 != tmp.1 && tmp.1.is_some()))
                        || !config.only_update_on_ip_change
                    {
                        ips = tmp;

                        if scan {
                            config = get_checked_config(matches.get_one("config"));
                        }
                        let result = update_records(&config, ips.0, ips.1);
                        if let Err(e) = result {
                            eprintln!("Error while updating domain : {e}")
                        }
                    }
                }
            }
        }
        _ => unreachable!(),
    };
}

fn update_records(
    config: &config::Config,
    v4: Option<Ipv4Addr>,
    v6: Option<Ipv6Addr>,
) -> Result<(), reqwest::Error> {
    for domain in &config.domains {
        for response in domain.update(v4, v6) {
            let address_update_text = match response.2 {
                IpType::V4 => "IPv4 address",
                IpType::V6 => "IPv6 address",
                IpType::Both => "IPv4 and IPv6 addresses",
            };
            match response.0?.error_for_status() {
                Ok(_) => {
                    println!("Updated {} for {}", address_update_text, response.1)
                }
                Err(e) => {
                    eprintln!(
                        "Error in updating {} {}, {e}",
                        response.1, address_update_text
                    )
                }
            }
        }
    }

    Ok(())
}

fn get_checked_config(conf_path: Option<&PathBuf>) -> config::Config {
    let config_path = conf_path.expect("Default value is implemented");

    let config_text = match fs::read_to_string(config_path) {
        Ok(text) => text,
        Err(error) => {
            eprintln!("Error, could not open config file: {error}");
            process::exit(1);
        }
    };

    let mut config: config::Config = match toml::from_str(&config_text) {
        Ok(unvalidated_config) => unvalidated_config,
        Err(error) => {
            eprintln!("Error, invalid config: {error}");
            process::exit(1);
        }
    };

    for domain in &mut config.domains {
        if let Some(defaults) = &config.defaults {
            domain.init_defaults(defaults)
        }
    }

    for domain in &config.domains {
        if let Err(error) = domain.check_config() {
            eprintln!("Error: {}", error);
            process::exit(1);
        }
    }
    config
}

fn print_addresses(v4: Option<Ipv4Addr>, v6: Option<Ipv6Addr>) {
    match v4 {
        Some(addr) => println!("Ipv4 address: {}", addr),
        None => println!("No public Ipv4 address found"),
    }
    match v6 {
        Some(addr) => println!("Ipv6 address: {}", addr),
        None => println!("No public Ipv6 address found"),
    }
}
