use std::net::{Ipv4Addr, Ipv6Addr};

use enum_dispatch::enum_dispatch;
use serde::Deserialize;

use crate::ips::IpType;

/// This trait must be implemented for all dns provider structs to allow for them to be updated.
#[enum_dispatch(Providers)]
pub(crate) trait Provider {
    /// Builds a url string to update the providers dns records aswell as the domain and record being updated to be displayed
    /// The provider should return an error code if something goes wrong, so that doesn't need to be handled on the provider level.
    /// Config sanity checks are already done beforehand, so there is no reason for this function to return an error or panic.
    fn update(
        &self,
        v4: Option<Ipv4Addr>,
        v6: Option<Ipv6Addr>,
    ) -> Vec<(
        Result<reqwest::blocking::Response, reqwest::Error>,
        String,
        IpType,
    )>;

    /// Checks the sanity of the config for the provider.
    /// This should be used to return errors if values are invalid or outside of the allowed range of the provider.
    fn check_config(&self) -> Result<(), crate::error::Error>;

    /// Sets default values as the provider's current values if it is not present for the current config
    fn init_defaults(&mut self, defaults: &crate::config::Defaults);
}

#[derive(Deserialize)]
#[serde(tag = "provider", rename_all = "lowercase")]
#[enum_dispatch]
pub(crate) enum Providers {
    Gandi(gandi::Gandi),
    DuckDNS(duckdns::DuckDNS),
    Dynv6(dynv6::Dynv6),
}

mod duckdns;
mod dynv6;
mod gandi;
