use std::{
    net::{Ipv4Addr, Ipv6Addr},
    option::Option,
};

use crate::{error::Error, ips::IpType};

use super::Provider;

#[derive(serde::Deserialize)]
pub(crate) struct DuckDNS {
    token: String,
    subdomains: Vec<String>,
    ip_type: Option<IpType>,
}

impl Provider for DuckDNS {
    fn update(
        &self,
        _v4: Option<Ipv4Addr>,
        v6: Option<Ipv6Addr>,
    ) -> Vec<(
        Result<reqwest::blocking::Response, reqwest::Error>,
        std::string::String,
        IpType,
    )> {
        let mut requests = vec![];

        for subdomain in self.subdomains.clone() {
            let client = reqwest::blocking::Client::new();
            requests.push((
                client.execute(
                    client
                        .get("https://www.duckdns.org/update")
                        .query(&[
                            Some(("domains", Some(subdomain.clone()))),
                            Some(("token", Some(self.token.clone()))),
                            match self
                                .ip_type
                                .expect("Should be overwritten by init_defaults")
                            {
                                IpType::Both => Some(("ipv6", v6.map(|x| x.to_string()))),
                                IpType::V6 => Some(("ip", v6.map(|x| x.to_string()))),
                                IpType::V4 => None,
                            },
                        ])
                        .build()
                        .expect("Reqwest Documentation does not state when this could fail"),
                ),
                subdomain + ".duckdns.org",
                self.ip_type.expect("Must be Some() due to init_defaults()"),
            ));
        }

        requests
    }

    fn check_config(&self) -> Result<(), Error> {
        Ok(())
    }

    fn init_defaults(&mut self, defaults: &crate::config::Defaults) {
        if self.ip_type.is_none() {
            self.ip_type = Some(defaults.ip_type)
        };
    }
}
