use std::net::{Ipv4Addr, Ipv6Addr};

use crate::ips::IpType;

use super::Provider;

#[derive(serde::Deserialize)]
pub(crate) struct Dynv6 {
    domains: Vec<String>,
    token: String,
    ip_type: Option<IpType>,
}

impl Provider for Dynv6 {
    fn update(
        &self,
        v4: Option<Ipv4Addr>,
        v6: Option<Ipv6Addr>,
    ) -> Vec<(
        Result<reqwest::blocking::Response, reqwest::Error>,
        String,
        IpType,
    )> {
        let mut requests = vec![];

        let ips = match self
            .ip_type
            .expect("Should be overwritten by init_defaults")
        {
            IpType::Both => (
                Some(("ipv6", v6.map(|x| x.to_string().clone()))),
                Some(("ipv4", v4.map(|x| x.to_string().clone()))),
            ),
            IpType::V6 => (Some(("ipv6", v6.map(|x| x.to_string().clone()))), None),
            IpType::V4 => (Some(("ipv4", v4.map(|x| x.to_string().clone()))), None),
        };

        for domain in &self.domains {
            let client = reqwest::blocking::Client::new();
            requests.push((
                client.execute(
                    client
                        .get("https://dynv6.com/api/update")
                        .query(&[
                            Some(("zone", Some(domain.clone()))),
                            Some(("token", Some(self.token.clone()))),
                            ips.0.clone(),
                            ips.1.clone(),
                        ])
                        .build()
                        .expect("Reqwest Documentation does not state when this could fail"),
                ),
                domain.clone(),
                self.ip_type.expect("Must be Some() due to init_defaults()"),
            ));
        }

        requests
    }

    fn check_config(&self) -> Result<(), crate::error::Error> {
        Ok(())
    }

    fn init_defaults(&mut self, defaults: &crate::config::Defaults) {
        if self.ip_type.is_none() {
            self.ip_type = Some(defaults.ip_type)
        };
    }
}
