use super::Provider;
use crate::error::Error;
use crate::{config::Defaults, ips::IpType};
use serde_json::json;
use std::net::{IpAddr, Ipv4Addr, Ipv6Addr};

#[derive(serde::Deserialize)]
pub(crate) struct Gandi {
    domain: String,
    key: String,
    subdomains: Option<Vec<String>>,
    ip_type: Option<IpType>,
    ttl: Option<u32>,
}

impl Provider for Gandi {
    fn update(
        &self,
        v4: Option<Ipv4Addr>,
        v6: Option<Ipv6Addr>,
    ) -> Vec<(
        Result<reqwest::blocking::Response, reqwest::Error>,
        std::string::String,
        IpType,
    )> {
        let mut requests = vec![];

        for ip in match self.ip_type.expect("Must be set due to init_defaults()") {
            IpType::Both => [v4.map(IpAddr::V4), v6.map(IpAddr::V6)],
            IpType::V4 => [v4.map(IpAddr::V4), None],
            IpType::V6 => [None, v6.map(IpAddr::V6)],
        }
        .into_iter()
        .flatten()
        {
            for subdomain in self.subdomains.clone().unwrap_or(vec![String::from("@")]) {
                let record = match ip {
                    IpAddr::V4(_) => "A",
                    IpAddr::V6(_) => "AAAA",
                };

                let client = reqwest::blocking::Client::new();
                requests.push((
                    client.execute(
                        client
                            .put(format!(
                            "https://api.gandi.net/v5/livedns/domains/{}/records/{subdomain}/{}",
                            &self.domain, &
                            record
                        ))
                            .header("Authorization", format!("Apikey {}", &self.key))
                            .body(
                                serde_json::to_string(
                                    &json!({ "rrset_values": [ip], "rrset_ttl": &self.ttl }),
                                )
                                .expect("Both values correctly serialize"),
                            )
                            .build()
                            .expect("Reqwest Documentation does not state when this could fail"),
                    ),
                    if subdomain == "@" {
                        self.domain.clone()
                    } else {
                        subdomain + "." + &self.domain
                    },
                    match ip {
                        IpAddr::V4(_) => IpType::V4,
                        IpAddr::V6(_) => IpType::V6,
                    },
                )) // response?.error_for_status()?;
            }
        }

        requests
    }

    fn check_config(&self) -> Result<(), Error> {
        if let Some(ttl) = self.ttl {
            if !(300..=2592000).contains(&ttl) {
                return Err(Error::GandiTTLRange(ttl));
            }
        }

        Ok(())
    }

    fn init_defaults(self: &mut Gandi, defaults: &Defaults) {
        if self.ip_type.is_none() {
            self.ip_type = Some(defaults.ip_type)
        };

        if self.ttl.is_none() {
            self.ttl = Some(defaults.ttl)
        };
    }
}
